@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.2.19
+# Version   : 0.2.22
 
 import  numpy                   as      np
 import  plotly.graph_objects    as      go
@@ -472,7 +472,7 @@ class AFM:
 
         self.def_sens = float(_def_sens)
         self.spr_const = float(_spr_const)
-        self.samps_line = int(_approach[0].shape[0])
+        self.samps_line = int(_params['Header Dump']['DataSet-Info']['Data points'])
 
         self.approach_raw = np.array(_approach, dtype=object)
         self.retract_raw = np.array(_withdraw, dtype=object)
@@ -1014,7 +1014,101 @@ class AFM:
         except:
             pass
 
-        print('Mean modulus = %(mean).2f MPa' % {'mean': np.mean(self.modulus)})
+        print('Mean modulus = %(mean).2f MPa (Hertz model)' % {'mean': np.mean(self.modulus)})
+        print('Standard deviation = %(std).2f MPa' % {'std': np.std(self.modulus)})
+
+    def calc_modulus_sneddon(
+        self,
+        poisson_ratio = 0.5,
+        opening_angle = 35,
+        plot_hist = False,
+    ):
+        '''
+        Funciton to calculate the modulus value for each curve using the Sneddon model.
+        A histogram of calculated values can be plotted by setting the 'plot_hist'
+        input to 'True'
+        '''
+        self.modulus = np.zeros(self.z_approach.shape[1])
+
+        for curve_number in range(0, self.z_approach.shape[1]):
+            _approach = self.approach[:,curve_number]
+            _z = self.z_approach[:,curve_number]
+
+            force = _approach[np.isfinite(_approach)][0] * 1e-9
+            depth = _z[np.isfinite(_approach)][0] * 1e-9
+
+            term_1 = 2/np.pi
+            term_2 = 1 / (1 - poisson_ratio ** 2)
+            term_3 = 1 / force
+            term_4 = np.tan(np.deg2rad(opening_angle))
+            term_5 = np.absolute(depth) ** 2
+
+            terms = term_1 * term_2 * term_3 * term_4 * term_5
+
+            self.modulus[curve_number] = (1 / terms) * 1e-6
+        
+        if plot_hist is True:
+            fig = go.Figure(data=[go.Histogram(x = self.modulus)])
+            fig.update_layout(
+                xaxis_title_text='Modulus (MPa)',
+                yaxis_title_text='Count',
+            )
+            fig.show()
+
+        print('')
+        try:
+            print(self.run_name)
+        except:
+            pass
+
+        print('Mean modulus = %(mean).2f MPa (Sneddon Model)' % {'mean': np.mean(self.modulus)})
+        print('Standard deviation = %(std).2f MPa' % {'std': np.std(self.modulus)})
+
+    def calc_modulus_pyramid(
+        self,
+        poisson_ratio = 0.5,
+        face_angle = 35,
+        plot_hist = False,
+    ):
+        '''
+        Funciton to calculate the modulus value for each curve using the four sided pyramid model.
+        A histogram of calculated values can be plotted by setting the 'plot_hist'
+        input to 'True'
+        '''
+        self.modulus = np.zeros(self.z_approach.shape[1])
+
+        for curve_number in range(0, self.z_approach.shape[1]):
+            _approach = self.approach[:,curve_number]
+            _z = self.z_approach[:,curve_number]
+
+            force = _approach[np.isfinite(_approach)][0] * 1e-9
+            depth = _z[np.isfinite(_approach)][0] * 1e-9
+
+            term_1 = 1/np.sqrt(2)
+            term_2 = 1 / (1 - poisson_ratio ** 2)
+            term_3 = 1 / force
+            term_4 = np.tan(np.deg2rad(face_angle))
+            term_5 = np.absolute(depth) ** 2
+
+            terms = term_1 * term_2 * term_3 * term_4 * term_5
+
+            self.modulus[curve_number] = (1 / terms) * 1e-6
+        
+        if plot_hist is True:
+            fig = go.Figure(data=[go.Histogram(x = self.modulus)])
+            fig.update_layout(
+                xaxis_title_text='Modulus (MPa)',
+                yaxis_title_text='Count',
+            )
+            fig.show()
+
+        print('')
+        try:
+            print(self.run_name)
+        except:
+            pass
+
+        print('Mean modulus = %(mean).2f MPa (Four sided pyramid)' % {'mean': np.mean(self.modulus)})
         print('Standard deviation = %(std).2f MPa' % {'std': np.std(self.modulus)})
 
     def overlay_modulus_hist(
